@@ -4,8 +4,8 @@
 
 ;; Author: Graham Marlow
 ;; Keywords: convenience
-;; Package-Version: 20251125.2011
-;; Package-Revision: 95720173db4b
+;; Package-Version: 20260227.2334
+;; Package-Revision: f97bda708771
 ;; Package-Requires: ((emacs "29.1"))
 ;; URL: https://github.com/mgmarlow/helix-mode
 
@@ -72,7 +72,9 @@
 ;;; Code:
 
 (require 'helix-core)
-(require 'helix-multiple-cursors)
+(when (locate-library "multiple-cursors")
+  (require 'helix-multiple-cursors)
+  (helix-multiple-cursors-setup))
 (require 'helix-jj)
 
 (provide 'helix)
